/*
 * arch/mips/dec/decstation.c
 *
 * Copyright (C) 1996,1997 Paul M. Antoine, Frieder Streffer, Harald Koerfgen,
 * and Michael Engel.
 *
 * Originally written for DS5000's by Paul Antoine.
 *
 * Modified for DS2100/3100's by Frieder Streffer, Harald Koerfgen, and
 * Michael Engel.
 *
 * FIXME: still plenty to do in this file, as we don't yet fully fill
 *        the boot info structure with DEC-specific tags.  The code is
 *	  also still too specific to the Personal Decstation 5000/2x and
 *	  DS2100/3100's - need to make sure other models are o.k. - PMA
 */

#include <asm/addrspace.h>
#include <asm/dec/prom.h>
#include <asm/bootinfo.h>
#include <asm/system.h>
#include <asm/mipsregs.h>
#include <asm/ptrace.h>

/* FIXME: what about other decstations? */
#include <asm/dec/kn01.h>
#include <asm/dec/kn02.h>
#include <asm/dec/kn02ba.h>
#include <asm/dec/kn02ca.h>

#include <asm/dec/interrupts.h>

/*
 * dec_setup: Called at boot from dec_entry() in boot.S to do 
 *            DECStation-specific setup, and to fill in the kernel argument
 *            tags.
 *
 * FIXME: I'm not sure all DEC workstations are correctly supported.  This
 *        code may not need to be here when booting off floppy or HD??
 */

unsigned long mach_mem_upper = 0;
unsigned long mach_mem_lower = 0;
unsigned long mips_dcache_size = 0;
unsigned long mips_icache_size = 0;
unsigned long rex_prom_magic = 0L; /* from boot.S */
unsigned long rex_prom_base = 0L; /* from boot.S */
unsigned long dec_sysid = 0L;
unsigned long pmax_get_memory_size(void);
unsigned long rex_get_memory_size(void);
unsigned long which_mips_cpu(void);
void dec_init_kn01(void);
void dec_init_kn02(void);
void dec_init_kn02ba(void);
void dec_init_kn02ca(void);
void dec_init_kn03(void);

char *dec_rtc_base = (char *)KN01_RTC_BASE;	/* Assume DS2100/3100 initially */
char *dec_console_type = "DS2100-Framebuffer";

int string_to_int(char *);

extern void cache_init(void);
extern void move_exception_code(void);
extern void which_prom(void);
extern int serial_console;		/* in console.c */
extern prom_call_t prom_call;		/* in dec_whichprom.c */

void dec_setup(void)
{
    unsigned long tag_data_dummy;
    unsigned char dec_cpunum, dec_systype, dec_firmrev, dec_etc;
    char *console_type;
    extern const char *linux_banner;

    /* Determine which PROM's we have (and therefore which machine we're on!) */
    which_prom();
    /* Now we can say hello! */
    prom_printf("\n\nLinux/MIPS DECStation Boot\n");
    prom_printf("Copyright (C) Paul M. Antoine 1995, 1996, 1997 and others, 1994, 1995, 1996, 1997\n\n");
    prom_printf("%s\n", linux_banner);

    mach_mem_lower = 0x80000000L;
    /* First we need the memory upper bound before we can add tag entries...
     * FIXME: may need to do a memory search routine for the 5000/2xx
     */
    if (rex_prom_magic != REX_PROM_MAGIC)
        mach_mem_upper = mach_mem_lower + pmax_get_memory_size();
    else
        mach_mem_upper = mach_mem_lower + rex_get_memory_size();

    prom_printf("Found memory size: %dkB\n",
		((mach_mem_upper - mach_mem_lower)/1024));
    
    /* First tag is always memory upper limit, right Stoned?? */
    (void)bi_TagAdd(tag_memupper, ULONGSIZE, &mach_mem_upper);
    mach_mem_lower = 0x80030000L;	/* FIXME: a kludge! */

    /* We're obviously one of the DEC machines */
    tag_data_dummy = MACH_GROUP_DEC;
    (void)bi_TagAdd(tag_machgroup, ULONGSIZE, &tag_data_dummy);

    /* Initialise and size the caches */
    cache_init();
    prom_printf("I-Cache size: %dkB\nD-Cache size: %dkB\n",
		mips_icache_size/1024, mips_dcache_size/1024);

    /*
     * Add the instruction cache size tags
     */
    tag_data_dummy = mips_icache_size;
    (void)bi_TagAdd(tag_icache_size, ULONGSIZE, &tag_data_dummy);

    /*
     * Add the data cache size
     */
    tag_data_dummy = mips_dcache_size;
    (void)bi_TagAdd(tag_dcache_size, ULONGSIZE, &tag_data_dummy);

    /* Now let's try to figure out what type of DECStation we are.  We
     * can do this by getting the system type in one of two ways: REX
     * proms have a suitable prom callback routine; non-REX prom
     * machines (which include the 2100/3100 models AND DS5000/2xx!)
     * proms store it in the 'systype' environment variable!.
     */
    if (rex_prom_magic != REX_PROM_MAGIC)
    {
	    dec_sysid = string_to_int(prom_getenv("systype"));
    }
    else
    {
	    dec_sysid = prom_getsysid();
	    if (dec_sysid == 0)
	    {
		    prom_printf("Zero sysid returned from PROMs! Assuming PMAX-like machine.\n");
		    dec_sysid == 1;
	    }
    }

    dec_cpunum = (dec_sysid & 0xff000000) >> 24;
    dec_systype = (dec_sysid & 0xff0000) >> 16;
    dec_firmrev = (dec_sysid & 0xff00) >> 8;
    dec_etc = dec_sysid & 0xff;

    /* FIXME: Don't know what firmware type is returned for 2100/3100's or 5000/2xx's */
    prom_printf("System has firmware type: ");
    if (dec_firmrev == 2)
      prom_printf("TCF0\n");
    else
      prom_printf("TCF1\n");

    /*
     * FIXME: This may not be an exhaustive list of DECStations/Servers!
     * Put all model-specific initialisation calls here.
     */
    prom_printf("This DECStation is a: ");
    switch(dec_systype) {
    case 1: /* DS2100/3100 Pmax */
	    prom_printf("DS2100/3100\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn01();
	    break;
    case 12: /* FIXME: Assume DS5100 MIPSMATE is the same as 3100 with R3K! */
	    prom_printf("DS5100\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn01();
	    break;
    case 2: /* DS5000 3max */
	    prom_printf("DS5000/200\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn02();
	    break;
    case 3: /* DS5000/100 3min */
	    prom_printf("DS5000/1x0\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn02ba();
	    break;
    case 4: /* DS5000/240 3max+ */
	    prom_printf("DS5000/1x0\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn03();
	    break;
    case 7: /* Personal DS5000/2x */
	    prom_printf("Personal DS5000/2x\n");
	    tag_data_dummy = MACH_DECSTATION;
	    dec_init_kn02ca();
	    break;
    case 5: /* DS5800 Isis */
	    prom_printf("DS5800\n");
	    tag_data_dummy = MACH_UNKNOWN;
	    break;
    case 6: /* DS5400 MIPSfair */
	    prom_printf("DS5000/1x0\n");
	    tag_data_dummy = MACH_UNKNOWN;
	    break;
    case 11: /* DS5500 MIPSfair-2 */
	    prom_printf("DS5500\n");
	    tag_data_dummy = MACH_UNKNOWN;
	    break;
    default:
	    prom_printf("unknown, id is: %x\n", dec_systype);
	    tag_data_dummy = MACH_UNKNOWN;
	    break;
    }

    /* Add the machine type */
    (void)bi_TagAdd(tag_machtype, ULONGSIZE, &tag_data_dummy);

    /* FIXME: Add the number of tlb entries - is this different for R4x00?? */
    tag_data_dummy = 64;
    (void)bi_TagAdd(tag_tlb_entries, ULONGSIZE, &tag_data_dummy);

    /* FIXME: should determine vram_base properly */
    if(dec_systype == 1)
      tag_data_dummy = 0xafc00000;
    else
      tag_data_dummy = 0xa8000000;
    (void)bi_TagAdd(tag_vram_base, ULONGSIZE, &tag_data_dummy);

    /* FIXME: dummy drive info tag */
    tag_data_dummy = 0;
    (void)bi_TagAdd(tag_drive_info, ULONGSIZE, &tag_data_dummy);

    /* FIXME: do we need a dummy tag at the end? */
    tag_data_dummy = 0;
    (void)bi_TagAdd(tag_dummy, 0, &tag_data_dummy);

    /* Register the PROM console for the moment */
    register_console(rex_printf);

    /* FIXME: Real console registration looks something like this... (this is
     * clearly unfinished!!)
     *
     * FIXME: Is "osconsole" the same variable on the DS2100/3100's??
     */
    console_type = prom_getenv("osconsole");
    if (console_type != NULL)
    {
	    prom_printf("Got the following for the osconsole env. variable: %s\n",
			console_type);
	    /* Big case statement goes here to set dec_console_type to the
	     * appropriate value for initialisation in con_type_init() in
	     * drivers/dec/char/deccons.c  In the meantime...
	     */
	    serial_console = 0;		/* assume graphics for the moment */
	    if ((console_type[1] == ',') || (console_type[0] == '*'))
	    {
		    prom_printf("Should set graphics console for DS5000 in slot %c, kbd/mouse in slot:%c\n",
				console_type[0], console_type[2]);
		    dec_console_type = "GraphicsConsole";
	    }
	    else if (console_type[0] == 's')
	    {
		    prom_printf("Should set serial console for DS5000\n");
		    serial_console = 1;	/* assume first serial port */
	    }
	    else
		    prom_printf("No idea what console to use!\n");
    }
} /* dec_setup */

/*
 * Machine-specific initialisation for kn01, aka Pmax, aka DS2100, DS3100,
 * and possibly also the DS5100.
 */
void dec_init_kn01(void)
{
	/*
	 * Setup some memory addresses. FIXME: probably incomplete!
	 */
	dec_rtc_base = (char *)KN01_RTC_BASE;
	/*
	 * Setup some interrupt vectors. FIXME: definitely incomplete!
	 */
	vector_table[KN01_RTC_IRQ] = dec_intr_rtc;
} /* dec_init_kn01 */

/*
 * Machine-specific initialisation for kn02, aka 3max, aka DS5000/2xx.
 */
void dec_init_kn02(void)
{
	/*
	 * Setup some memory addresses. FIXME: probably incomplete!
	 */
	dec_rtc_base = (char *)KN02_RTC_BASE;
	/*
	 * Setup some interrupt vectors. FIXME: definitely incomplete!
	 */
	vector_table[KN02_RTC_IRQ] = dec_intr_rtc;
} /* dec_init_kn02 */

/*
 * Machine-specific initialisation for kn02ba, aka 3min, aka DS5000/1xx.
 */
void dec_init_kn02ba(void)
{
/* FIXME: Don't know how to set one of these up!
 */
	prom_printf("Eeeek! Don't know how to set one of these up!\n");
	prom_halt();
} /* dec_init_kn02ba */

/*
 * Machine-specific initialisation for kn02ca, aka maxine, aka DS5000/2x.
 */
void dec_init_kn02ca(void)
{
	/*
	 * Setup some memory addresses. FIXME: probably incomplete!
	 */
	dec_rtc_base = (char *)KN02CA_RTC_BASE;
	/*
	 * Setup some interrupt vectors. FIXME: definitely incomplete!
	 */
	vector_table[KN02CA_RTC_IRQ] = dec_intr_rtc;
	vector_table[KN02CA_HALT_IRQ] = kn02ca_halt;
} /* dec_init_kn02ca */

/*
 * Machine-specific initialisation for kn03, aka 3max+, aka DS5000/240.
 */
void dec_init_kn03(void)
{
/* FIXME: Don't know how to set one of these up!
 */
	prom_printf("Eeeek! Don't know how to set one of these up!\n");
	prom_halt();
} /* dec_init_kn03 */

unsigned char rtc_read_data(unsigned long addr)
{
    return(dec_rtc_base[addr * 4]);
} /* rtc_read_data */

void rtc_write_data(unsigned char data, unsigned long addr)
{
    dec_rtc_base[addr * 4] = data;
} /* rtc_read_data */

/*
 * Routines to detect memory size in DECstations.  Note that on the
 * DS2100/3100's we must probe around in memory, so we might as well do
 * it for all of them.  We may go back to using the REX PROM routines on
 * those machines that have them as it's probably quicker, particularly if
 * there's lots of memory.
 */

volatile unsigned long mem_err=0;	/* So we know an error occured */

/*
 * Handle the inevitable DBE when we eventually run off the end of real
 * memory.
 */
void handle_dbe_early(struct pt_regs *regs)
{
   ++mem_err;
   regs->regs[4]-=4096;
}

/*
 * Probe memory in 4MB chunks, waiting for an error to tell us we've fallen
 * off the end of real memory.  Only suitable for the 2100/3100's (PMAX).
 */
unsigned long pmax_get_memory_size(void) 
{
   unsigned long save_exception=0;
   unsigned long num_pages=0;
   volatile unsigned char *memory_page;
   extern long dbe_board_handler;
   extern void handle_dbe(void);
   extern void check_memory(unsigned char *, int);

   /* initialize trap code */
   set_except_vector(7, handle_dbe); 

   /* alter trap #7 (Data Bus Error) to our own routine */
   save_exception = dbe_board_handler;
   dbe_board_handler = (unsigned long) (handle_dbe_early);

   /* store and read unmapped and uncached (KSEG1)
    * DECstations have at least 4MB RAM
    * Assume less than 480MB of RAM, as this is max for 5000/2xx
    * FIXME this should be replaced by the first free page!
    */
   for(memory_page = (unsigned char *)(KSEG1 + 0x400000);
       (memory_page < (unsigned char *)(KSEG1+ 0x1E000000)) && (mem_err == 0);
       memory_page += 4096)
   {  
     check_memory(memory_page,1);
     num_pages++;
   }

   dbe_board_handler = save_exception;

   return (num_pages - 1 + 1024) * 4 * 1024;
} /* pmax_get_memory_size() */

/*
 * Use the REX prom calls to get hold of the memory bitmap, and thence
 * determine memory size.
 */
unsigned long rex_get_memory_size(void)
{
    int i, bitmap_size;
    unsigned long mem_size = 0;
    struct rex_bitmap {
      int		pagesize;
      unsigned char	bitmap[64*1024*1024 - 4];
    } *bm;

    /* some free 64k */
    bm = (struct rex_bitmap *)0x8002f000;

    bitmap_size = prom_getbitmap(bm);

    for (i = 0; i < bitmap_size; i++)
    {
      /* FIXME: very simplistically only add full sets of pages */
      if (bm->bitmap[i] == 0xff)
	  mem_size += (8 * bm->pagesize);
    }
    return(mem_size);
} /* rex_get_memory_size */

/*
 * Both of the following routines come from Mach, and are temporary until such
 * time as I get around to writing ASM versions!
 */

/*
 *	Object:
 *		string_to_int			EXPORTED function
 *
 *	Convert a string into an integer.
 *	Understands decimal (default), octal and hex numbers
 *	like MIPS' atob(), for compat reasons.
 *	It also understands Ada-like numbers of the form
 *		#base#digits
 *	Defaults to 0 for all strange strings.
 *
 */
int string_to_int(char *s)
{
	static unsigned      char_to_uint(register char);
	char            c;
	unsigned        base = 10, d;
	int             neg = 1, val = 0;

	if ((s == 0) || ((c = *s++) == 0))
		goto out;

	/* skip spaces if any */
	while ((c == ' ') || (c == '\t'))
		c = *s++;

	/* parse sign, allow more than one (compat) */
	while (c == '-') {
		neg = -neg;
		c = *s++;
	}

	/* parse base specification, if any */
	if ((c == '0') || (c == '#')) {
		if (c == '0') {
			c = *s++;
			switch (c) {
			case 'X':
			case 'x':
				base = 16;
				break;
			case 'B':
			case 'b':
				base = 2;
				break;
			default:
				base = 8;
				break;
			}
		} else {
			c = *s++;
			while ((d = char_to_uint(c)) < base) {
				val *= base;
				val += d;
				c = *s++;
			}
			base = val;
			val = 0;
		}
		c = *s++;	/* forgiving of not '#' */
	}

	/* parse number proper */
	while ((d = char_to_uint(c)) < base) {
		val *= base;
		val += d;
		c = *s++;
	}
	val *= neg;
out:
	return val;	
}


/*
 *	Object:
 *		char_to_uint			LOCAL function
 *
 *	Convert a character into an integer.
 *	Besides the numbers 0..9 understands all
 *	letters of the alphabet, starting at a=10.
 *	Case insensitive, returns -1 for bogus characters.
 *
 */
static unsigned int char_to_uint(register char c)
{
	if ((c >= '0') && (c <= '9'))
		return c - '0';

	if ((c >= 'a') && (c <= 'z'))
		return c - 'a' + 10;

	if ((c >= 'A') && (c <= 'Z'))
		return c - 'A' + 10;

	return 0xffffffff;
}


