/*  
 * Miscellaneous definitions used to call the routines contained in the boot
 * PROMs on various models of DECSTATION's.
 *
 * Much of this file is from code found in the Mach Operating System.
 *
 * Modifications to this file (originally called asm_misc.h are
 * Copyright (C) 1995,1996,1997 by Paul M. Antoine, with some code and
 * definitions curteousy of Chris Fraser.
 *
 * Other parts of this file are therefore:
 *
 * Copyright (c) 1991,1990,1989 Carnegie Mellon University
 * All Rights Reserved.
 *
 * And as such I must quote the following copyright, that applies to those
 * portions of this file so covered:
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

#ifndef __ASM_DEC_PROM_H 
#define __ASM_DEC_PROM_H 

#ifndef __LANGUAGE_ASSEMBLY__

/*
 * PROM callback interface structure to hide differences between earlier
 * non-REX boot PROM's for systems such as the DS2100/3100's and DS5000/2xx's,
 * and newer REX PROM's found in DS5000/1xx's and DS5000/2x's (which comply
 * with the ARC specification for TurboChannel systems).
 */
typedef struct {
	void	(*halt)(void);
	int	(*open)(char*, int);
	int	(*lseek)(int, long, int);
	int	(*read)(int, void *, int);
	int	(*close)(int);
	int	(*gets)(char*);
	int	(*printf)(char*, ...);
	char	*(*getenv)(char*);
	/*
	 * The following are NOT available on DS2100/3100's or 5000/2xx's,
	 * but are placed here for use by DS5000/1xx's and DS5000/2x's.
	 */
	unsigned long (*getbitmap)(void *);
	unsigned long (*getsysid)(void);
} prom_call_t;

extern prom_call_t prom_call;

/*
 * Save a bit of typing...
 */
#define	prom_halt	(*prom_call.halt)
#define	prom_open	(*prom_call.open)
#define	prom_lseek	(*prom_call.lseek)
#define	prom_read	(*prom_call.read)
#define	prom_close	(*prom_call.close)
#define	prom_gets	(*prom_call.gets)
#define	prom_printf	(*prom_call.printf)
#define	prom_getenv	(*prom_call.getenv)
#define	prom_getbitmap	(*prom_call.getbitmap)
#define	prom_getsysid	(*prom_call.getsysid)

/*
 * PMAX/3MAX PROM routine prototypes for DS2100/3100's and DS5000/2xx's
 */
extern void	pmax_halt(void);
extern int	pmax_open(char*, int);
extern int	pmax_lseek(int, long, int);
extern int	pmax_read(int, void *, int);
extern int	pmax_close(int);
extern int	pmax_gets(char*);
extern int	pmax_printf(char*, ...);
extern char	*pmax_getenv(char*);
extern void	pmax_dummy(void);

/*
 * 3MIN/MAXINE PROM routine prototypes for DS5000/1xx's and DS5000/2x's, and
 * potentially other machines with TurboChannel and ARC-compliance (are there
 * any of these??? - PMA)
 */
extern void		rex_halt(void);
extern int		rex_open(char*, int);
extern int		rex_lseek(int, long, int);
extern int		rex_read(int, void *, int);
extern int		rex_close(int);
extern int		rex_gets(char*);
extern int		rex_printf(char*, ...);
extern char		*rex_getenv(char*);
extern unsigned long	rex_getbitmap(void *);
extern unsigned long	rex_getsysid(void);

#endif	 __LANGUAGE_ASSEMBLY__

/*
 * PMAX/3MAX PROM entry points for DS2100/3100's and DS5000/2xx's.  Many of
 * these will work for MIPSen as well!
 */
#define VEC_RESET		0xBFC00000		/* Prom base address */
#define	PMAX_PROM_ENTRY(x)	(VEC_RESET+((x)*8))	/* Prom jump table */

#define	PMAX_PROM_HALT		PMAX_PROM_ENTRY(2)	/* valid on MIPSen */
#define	PMAX_PROM_AUTOBOOT	PMAX_PROM_ENTRY(5)	/* valid on MIPSen */
#define	PMAX_PROM_OPEN		PMAX_PROM_ENTRY(6)
#define	PMAX_PROM_READ		PMAX_PROM_ENTRY(7)
#define	PMAX_PROM_CLOSE		PMAX_PROM_ENTRY(10)
#define	PMAX_PROM_LSEEK		PMAX_PROM_ENTRY(11)
#define	PMAX_PROM_PUTCHAR	PMAX_PROM_ENTRY(13)	/* 12 on MIPSen */
#define	PMAX_PROM_GETS		PMAX_PROM_ENTRY(15)
#define	PMAX_PROM_PRINTF	PMAX_PROM_ENTRY(17)
#define	PMAX_PROM_GETENV	PMAX_PROM_ENTRY(33)	/* valid on MIPSen */

/*
 * Magic number indicating REX PROM available on DECSTATION.  Found in
 * register a2 on transfer of control to program from PROM.
 */
#define	REX_PROM_MAGIC		0x30464354

/*
 * 3MIN/MAXINE PROM entry points for DS5000/1xx's and DS5000/2x's, and
 * potentially other machines with TurboChannel and ARC-compliance (are there
 * any of these??? - PMA)
 */
#define REX_PROM_GETBITMAP	0x84	/* get mem bitmap */
#define REX_PROM_GETCHAR	0x24	/* getch() */
#define REX_PROM_PUTCHAR	0x13	/* putch() */
#define REX_PROM_HALT		0x9c	/* halt the system */
#define REX_PROM_PRINTF		0x30	/* printf() */
#define	REX_PROM_GETS		0x28	/* gets() */
#define REX_PROM_PUTS		0x2c	/* puts() */
#define REX_PROM_SLOTADDR	0x6c	/* slotaddr */
#define REX_PROM_GETENV		0x64	/* get env. variable */
#define REX_PROM_GETSYSID	0x80	/* get system id */
#define REX_PROM_OPEN		0x54	/* open() */
#define REX_PROM_READ		0x58	/* read() */
#define	REX_PROM_PROM_REX	0xac	/* run a REX script? */
/*#define	REX_PROM_CLOSE		notimpl */
/*#define	REX_PROM_LSEEK		notimpl */

#endif /* __ASM_DEC_PROM_H */

